/*
** repl.c -- reply to a message
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <h/mh.h>
#include <h/utils.h>


static struct swit switches[] = {
#define GROUPSW  0
	{ "group", 0 },
#define NGROUPSW  1
	{ "nogroup", 0 },
#define ANNOSW  2
	{ "annotate", 0 },
#define NANNOSW  3
	{ "noannotate", 0 },
#define CCSW  4
	{ "cc all|to|cc|me", 0 },
#define NCCSW  5
	{ "nocc type", 0 },
#define EDITRSW  6
	{ "editor editor", 0 },
#define NEDITSW  7
	{ "noedit", 0 },
#define FCCSW  8
	{ "fcc folder", 0 },
#define FILTSW  9
	{ "filter filterfile", 0 },
#define FORMSW  10
	{ "form formfile", 0 },
#define FRMTSW  11
	{ "format", 5 },
#define NFRMTSW  12
	{ "noformat", 7 },
#define INPLSW  13
	{ "inplace", 0 },
#define NINPLSW  14
	{ "noinplace", 0 },
#define MIMESW  15
	{ "mime", 0 },
#define NMIMESW  16
	{ "nomime", 0 },
#define QURYSW  17
	{ "query", 0 },
#define NQURYSW  18
	{ "noquery", 0 },
#define WHATSW  19
	{ "whatnowproc program", 0 },
#define NWHATSW  20
	{ "nowhatnowproc", 0 },
#define WIDTHSW  21
	{ "width columns", 0 },
#define VERSIONSW  22
	{ "version", 0 },
#define HELPSW  23
	{ "help", 0 },
#define FILESW  24
	{ "file file", 4 },  /* interface from msh */

#ifdef MHE
#define BILDSW  25
	{ "build", 5 },  /* interface from mhe */
#endif

	{ NULL, 0 }
};

static struct swit ccswitches[] = {
#define CTOSW  0
	{ "to", 0 },
#define CCCSW  1
	{ "cc", 0 },
#define CMESW  2
	{ "me", 0 },
#define CALSW  3
	{ "all", 0 },
	{ NULL, 0 }
};

short ccto = -1;  /* global for replsbr */
short cccc = -1;
short ccme = -1;
short querysw = 0;

short outputlinelen = OUTPUTLINELEN;
short groupreply = 0;  /* Is this a group reply? */

int mime = 0;  /* include original as MIME part */
char *form   = NULL;  /* form (components) file */
char *filter = NULL;  /* message filter file */
char *fcc = NULL;  /* folders to add to Fcc: header */


/*
** prototypes
*/
void docc(char *, int);


int
main(int argc, char **argv)
{
	int anot = 0, inplace = 1;
	int nedit = 0, nwhat = 0;
	char *cp, *cwd, *dp, *maildir, *file = NULL;
	char *folder = NULL, *msg = NULL;
	char *ed = NULL, drft[BUFSIZ], buf[BUFSIZ];
	char **argp, **arguments;
	struct msgs *mp = NULL;
	FILE *in;

#ifdef MHE
	int buildsw = 0;
#endif /* MHE */

#ifdef LOCALE
	setlocale(LC_ALL, "");
#endif
	invo_name = mhbasename(argv[0]);

	/* read user profile/context */
	context_read();

	arguments = getarguments(invo_name, argc, argv, 1);
	argp = arguments;

	while ((cp = *argp++)) {
		if (*cp == '-') {
			switch (smatch(++cp, switches)) {
			case AMBIGSW:
				ambigsw(cp, switches);
				done(1);
			case UNKWNSW:
				adios(NULL, "-%s unknown", cp);

			case HELPSW:
				snprintf(buf, sizeof(buf), "%s: [+folder] [msg] [switches]", invo_name);
				print_help(buf, switches, 1);
				done(0);
			case VERSIONSW:
				print_version(invo_name);
				done(1);

			case GROUPSW:
				groupreply++;
				continue;
			case NGROUPSW:
				groupreply = 0;
				continue;

			case ANNOSW:
				anot++;
				continue;
			case NANNOSW:
				anot = 0;
				continue;

			case CCSW:
				if (!(cp = *argp++) || *cp == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				docc(cp, 1);
				continue;
			case NCCSW:
				if (!(cp = *argp++) || *cp == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				docc(cp, 0);
				continue;

			case EDITRSW:
				if (!(ed = *argp++) || *ed == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				nedit = 0;
				continue;
			case NEDITSW:
				nedit++;
				continue;

			case WHATSW:
				if (!(whatnowproc = *argp++) ||
						*whatnowproc == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				nwhat = 0;
				continue;
#ifdef MHE
			case BILDSW:
				buildsw++;  /* fall... */
#endif /* MHE */
			case NWHATSW:
				nwhat++;
				continue;

			case FCCSW:
				if (!(cp = *argp++) || *cp == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				dp = NULL;
				if (*cp == '@')
					cp = dp = getcpy(expandfol(cp));
				if (fcc)
					fcc = add(", ", fcc);
				fcc = add(cp, fcc);
				if (dp)
					free(dp);
				continue;

			case FILESW:
				if (file)
					adios(NULL, "only one file at a time!");
				if (!(cp = *argp++) || *cp == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				file = getcpy(expanddir(cp));
				continue;
			case FILTSW:
				if (!(cp = *argp++) || *cp == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				filter = getcpy(etcpath(cp));
				mime = 0;
				continue;
			case FORMSW:
				if (!(form = *argp++) || *form == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				continue;

			case FRMTSW:
				filter = getcpy(etcpath(mhlreply));
				mime = 0;
				continue;
			case NFRMTSW:
				filter = NULL;
				continue;

			case INPLSW:
				inplace++;
				continue;
			case NINPLSW:
				inplace = 0;
				continue;

			case MIMESW:
				mime++;
				filter = NULL;
				continue;
			case NMIMESW:
				mime = 0;
				continue;

			case QURYSW:
				querysw++;
				continue;
			case NQURYSW:
				querysw = 0;
				continue;

			case WIDTHSW:
				if (!(cp = *argp++) || *cp == '-')
					adios(NULL, "missing argument to %s",
							argp[-2]);
				if ((outputlinelen = atoi(cp)) < 10)
					adios(NULL, "impossible width %d",
							outputlinelen);
				continue;
			}
		}
		if (*cp == '+' || *cp == '@') {
			if (folder)
				adios(NULL, "only one folder at a time!");
			else
				folder = getcpy(expandfol(cp));
		} else {
			if (msg)
				adios(NULL, "only one message at a time!");
			else
				msg = cp;
		}
	}

	if (ccto == -1)
		ccto = groupreply;
	if (cccc == -1)
		cccc = groupreply;
	if (ccme == -1)
		ccme = groupreply;

	cwd = getcpy(pwd());

	if (file && (msg || folder))
		adios(NULL, "can't mix files and folders/msgs");

#ifdef MHE
	strncpy(drft, buildsw ? toabsdir("reply")
		: m_draft("new"), sizeof(drft));
#else
	strncpy(drft, m_draft("new"), sizeof(drft));
#endif /* MHE */

	if (file) {
		/*
		** We are replying to a file.
		*/
		anot = 0;  /* we don't want to annotate a file */
	} else {
		/*
		** We are replying to a message.
		*/
		if (!msg)
			msg = "cur";
		if (!folder)
			folder = getcurfol();
		maildir = toabsdir(folder);

		if (chdir(maildir) == NOTOK)
			adios(maildir, "unable to change directory to");

		/* read folder and create message structure */
		if (!(mp = folder_read(folder)))
			adios(NULL, "unable to read folder %s", folder);

		/* check for empty folder */
		if (mp->nummsg == 0)
			adios(NULL, "no messages in %s", folder);

		/* parse the message range/sequence/name and set SELECTED */
		if (!m_convert(mp, msg))
			done(1);
		seq_setprev(mp);  /* set the previous-sequence */

		if (mp->numsel > 1)
			adios(NULL, "only one message at a time!");

		context_replace(curfolder, folder); /* update current folder */
		seq_setcur(mp, mp->lowsel);  /* update current message  */
		seq_save(mp);  /* synchronize sequences   */
		context_save();  /* save the context file   */
	}

	msg = file ? file : getcpy(m_name(mp->lowsel));

	if ((in = fopen(msg, "r")) == NULL)
		adios(msg, "unable to open");

	/* find form (components) file */
	if (!form) {
		if (groupreply)
			form = etcpath(replgroupcomps);
		else
			form = etcpath(replcomps);
	}

	replout(in, msg, drft, mp, outputlinelen, mime, form, filter, fcc);
	fclose(in);

	if (nwhat)
		done(0);
	what_now(ed, nedit, NOUSE, drft, msg, 0, mp, anot ? "Replied" : NULL,
			inplace, cwd);
	done(1);
	return 1;
}

void
docc(char *cp, int ccflag)
{
	switch (smatch(cp, ccswitches)) {
	case AMBIGSW:
		ambigsw(cp, ccswitches);
		done(1);
	case UNKWNSW:
		adios(NULL, "-%scc %s unknown", ccflag ? "" : "no", cp);

	case CTOSW:
		ccto = ccflag;
		break;

	case CCCSW:
		cccc = ccflag;
		break;

	case CMESW:
		ccme = ccflag;
		break;

	case CALSW:
		ccto = cccc = ccme = ccflag;
		break;
	}
}
